set global log_bin_trust_function_creators=1;

DROP TRIGGER IF EXISTS product_insert_trigger;
DROP TRIGGER IF EXISTS product_update_trigger;
DROP TRIGGER IF EXISTS product_delete_trigger;

DROP TABLE IF EXISTS customer;
DROP TABLE IF EXISTS employee;
DROP TABLE IF EXISTS employee_department;
DROP TABLE IF EXISTS department;
DROP TABLE IF EXISTS product;
DROP TABLE IF EXISTS product_change_history;
DROP TABLE IF EXISTS sale;
DROP TABLE IF EXISTS promotion;

--
-- customer
--

CREATE TABLE customer (
  id bigint auto_increment primary key not null,
  customer_id varchar(25) unique not null,
  first_name varchar(100),
  last_name varchar(100),
  email varchar(255),
  birth_date varchar(255),
  phone varchar(25),
  address varchar(255),
  city varchar(255),
  state varchar(2),
  zip varchar(10),
  sales_rep_id varchar(25),
  created timestamp not null default current_timestamp,
  updated timestamp not null default current_timestamp on update current_timestamp
);

CREATE INDEX customer_updated_idx ON customer (updated);

--
-- employee
--

CREATE TABLE employee (
  id bigint auto_increment primary key not null,
  employee_id varchar(25) unique not null,
  first_name varchar(100),
  last_name varchar(100),
  email varchar(255),
  birth_date date,
  phone varchar(25),
  address varchar(255),
  city varchar(255),
  state varchar(2),
  zip varchar(10),
  hire_date date,
  primary_department_id varchar(25),
  manager_employee_id varchar(25),
  classification_code varchar(25),
  created timestamp not null default current_timestamp,
  updated timestamp not null default current_timestamp on update current_timestamp
);

CREATE INDEX employee_updated_idx ON employee (updated);


--
-- department
--

CREATE TABLE department (
  id bigint auto_increment primary key not null,
  department_id varchar(25) unique not null,
  name varchar(100) not null,
  description varchar(4000),
  created timestamp not null default current_timestamp,
  updated timestamp not null default current_timestamp on update current_timestamp
);

CREATE INDEX department_updated_idx ON department (updated);


--
-- employee department
--

CREATE TABLE employee_department (
  employee_id varchar(25) not null,
  department_id varchar(25) not null,
  employee_role varchar(100) not null,
  created timestamp not null default current_timestamp,
  updated timestamp not null default current_timestamp on update current_timestamp,
  primary key(employee_id, department_id)
);

CREATE INDEX employee_department_updated_idx ON employee_department (updated);


--
-- products (change table method)
--

CREATE TABLE product (
  id bigint auto_increment primary key not null,
  product_id varchar(25) unique not null,
  name varchar(100) not null,
  description varchar(4000),
  price decimal(10,2),
  department_id varchar(25),
  manufacturer_id varchar(25),
  created timestamp not null default current_timestamp
);

CREATE TABLE product_change_history (
  product_change_history_id bigint auto_increment primary key not null,
  id bigint not null,
  product_id varchar(25) not null,
  change_type varchar(20) not null,
  change_time timestamp not null default current_timestamp
);

CREATE TRIGGER product_insert_trigger AFTER INSERT ON product FOR EACH ROW
    INSERT INTO product_change_history(id, product_id, change_type)
     VALUES (NEW.id, NEW.product_id, 'INSERT');

CREATE TRIGGER product_update_trigger AFTER UPDATE ON product FOR EACH ROW
    INSERT INTO product_change_history(id, product_id, change_type)
    VALUES (NEW.id, NEW.product_id, 'UPDATE');

CREATE TRIGGER product_delete_trigger AFTER DELETE ON product FOR EACH ROW
    INSERT INTO product_change_history(id, product_id, change_type)
    VALUES (OLD.id, OLD.product_id, 'DELETE');

--
-- purchase_order
--

CREATE TABLE purchase_order (
   id bigint auto_increment primary key not null,
   purchase_order_id varchar(25) unique not null,
   customer_id varchar(25) not null,
   employee_id varchar(25) not null,
   promotion_id varchar(25),
   description varchar(4000),
   total decimal(10,2),
   created timestamp not null default current_timestamp,
   updated timestamp not null default current_timestamp on update current_timestamp
);

CREATE INDEX purchase_order_updated_idx ON purchase_order (updated);

--
-- sale
--

CREATE TABLE sale (
  id bigint auto_increment primary key not null,
  sale_id varchar(25) unique not null,
  purchase_order_id varchar(25) not null,
  product_id varchar(25) not null,
  quantity int not null,
  unit_price decimal(10,2) not null,
  discount decimal(10,2),
  total decimal(10,2) not null,
  description varchar(4000),
  created timestamp not null default current_timestamp,
  updated timestamp not null default current_timestamp on update current_timestamp
);

CREATE INDEX sale_updated_idx ON sale (updated);

--
-- promotion
--

CREATE TABLE promotion (
  id bigint auto_increment primary key not null,
  promotion_id varchar(25) unique not null,
  description varchar(4000) not null,
  discount decimal(10,2) not null,
  created timestamp not null default current_timestamp,
  updated timestamp not null default current_timestamp on update current_timestamp
);

CREATE INDEX promotion_updated_idx ON promotion (updated);

--
-- manufacturer
--

CREATE TABLE manufacturer (
  id bigint auto_increment primary key not null,
  manufacturer_id varchar(25) unique not null,
  name varchar(255) not null,
  location varchar(4000),
  created timestamp not null default current_timestamp,
  updated timestamp not null default current_timestamp on update current_timestamp
);

CREATE INDEX manufacturer_updated_idx ON manufacturer (updated);