import os

from functions import *

assert sys.version_info >= (3, 7)

#
# Configure the biz demo databases.
#
# Waits for each database to become available by running a simple "SELECT READY" command.
# In the case of Oracle, as sqlplus is available before the database has started,
# it checks for the instance to be OPEN as well.
#
# Once the database is available, the DB-specific initialization script is copied
# to the containers and is run.
#

script_path = os.path.dirname(os.path.realpath(__file__))
sql_directory = script_path + "/../adaptors/sql/"
k8s_directory = script_path + "/../adaptors/kubernetes/"

postgresql_db = "north-america-postgres-db"
postgresql_asia_db = "asia-postgres-db"
postgresql_wait = postgresql_cmd(["-c", "SELECT 'READY'"])
postgresql_wait_input = None
postgresql_run = postgresql_cmd(["-f", "/init.sql"])
postgresql_sql_file = sql_directory + "postgresql.sql"

mysql_db = "europe-east-mysql-db"
mysql_wait = mysql_cmd()
mysql_wait_input = "SELECT 'READY';"
mysql_run = ["sh", "-c", " ".join(mysql_cmd()) + " < /init.sql"]
mysql_sql_file = sql_directory + "mysql.sql"

oracle_db = "asia-oracle-db"
oracle_wait = oracle_cmd()
oracle_wait_input = "SELECT 'READY' FROM v$instance WHERE STATUS = 'OPEN';"
oracle_run = oracle_cmd("@/init.sql")
oracle_sql_file = sql_directory + "oracle.sql"

sqlserver_db = "europe-west-sql-server-db"
sqlserver_wait = sqlserver_cmd(["-Q", "SELECT 'READY'"])
sqlserver_wait_input = None
sqlserver_run = sqlserver_cmd(["-i", "init.sql"])
sqlserver_sql_file = sql_directory + "sqlserver.sql"


def init_one_local_db(namespace, name, wait_cmd, wait_input, cp_file, run_cmd):
    """Initialize one locally running database"""

    print("Initializing " + name)
    
    # wait for ready
    cmd = ["docker", "exec", name] + wait_cmd
    if wait_input is not None:
        cmd.insert(2, "-i")
    wait_for_ready(cmd, wait_input)

    # copy the file to /init.sql 
    exec_command(["docker", "cp", cp_file, name + ":/init.sql"])

    # execute the command to run the sql file
    exec_command(["docker", "exec", name] + run_cmd)


def init_one_aws_demo_db(namespace, name, wait_cmd, wait_input, cp_file, run_cmd):
    """Initialize one database running in AWS demo"""

    print("Initializing " + name)
    
    # wait for ready
    cmd = ["kubectl", "exec", "deployments/" + name, "--namespace=" + namespace, "--"] + wait_cmd
    if wait_input is not None:
        cmd.insert(2, "-i")
    wait_for_ready(cmd, wait_input)
    pod = k8s_get_pod_name(name, namespace)
    
    # copy the file to /init.sql
    k8s_cp(pod, cp_file, "/init.sql", namespace)

    # execute the command to run the sql file
    k8s_exec(pod, run_cmd, namespace)


def __do_init(func_name, namespace, adaptor_number):
    """Initialize the databases with the given function name"""

    func = getattr(sys.modules[__name__], func_name)

    if adaptor_number is None or adaptor_number == "1":
        func(
            namespace=namespace,
            name=postgresql_asia_db,
            wait_cmd=postgresql_wait,
            wait_input=postgresql_wait_input,
            cp_file=postgresql_sql_file,
            run_cmd=postgresql_run
        )

    if adaptor_number is None or adaptor_number == "2":
        func(
            namespace=namespace,
            name=sqlserver_db,
            wait_cmd=sqlserver_wait,
            wait_input=sqlserver_wait_input,
            cp_file=sqlserver_sql_file,
            run_cmd=sqlserver_run
        )

    if adaptor_number is None or adaptor_number == "3":
        func(
            namespace=namespace,
            name=postgresql_db,
            wait_cmd=postgresql_wait,
            wait_input=postgresql_wait_input,
            cp_file=postgresql_sql_file,
            run_cmd=postgresql_run
        )

    if adaptor_number is None or adaptor_number == "4":
        func(
            namespace=namespace,
            name=mysql_db,
            wait_cmd=mysql_wait,
            wait_input=mysql_wait_input,
            cp_file=mysql_sql_file,
            run_cmd=mysql_run
        )


def start_databases_local():
    """Start the databases locally in docker"""

    exec_command(["docker", "compose", "-f", script_path + "/../adaptors/docker/docker-compose.yml", "pull"])
    exec_command(["docker", "compose", "-p", "adaptors_databases", "-f",
                     script_path + "/../adaptors/docker/docker-compose.yml", "up", "-d"])

def stop_databases_local():
    """Stop the locally running databases"""
    exec_command(["docker", "compose", "-p", "adaptors_databases", "-f",
                 script_path + "/../adaptors/docker/docker-compose.yml", "down", "-v"])


def start_databases_aws_demo(namespace):
    """Start the databases in the AWS demo"""

    print("Starting Mongo and Mongo Express ...")
    k8s_apply(k8s_directory + "mongo.yml", namespace)

    print("Starting Asia - Oracle")
    k8s_apply(k8s_directory + "asia-database.yml", namespace)

    print("Starting Europe West - SQL Server")
    k8s_apply(k8s_directory + "europe-west-database.yml", namespace)

    print("Starting Europe East - MySQL")
    k8s_apply(k8s_directory + "europe-east-database.yml", namespace)

    print("Starting North America - PostgreSQL")
    k8s_apply(k8s_directory + "north-america-database.yml", namespace)


def stop_databases_aws_demo(namespace):
    """Stop the databases in the AWS demo"""

    apps = ["north-america-postgres-db", "europe-west-sql-server-db", "europe-east-mysql-db",
            "asia-oracle-db", "mongo", "mongo-express"]

    k8s_drain(apps, namespace)


def init_local(adaptor_number):
    """Initialize the locally running databases"""
    __do_init("init_one_local_db", "", adaptor_number)


def init_aws_demo(namespace):
    """Initialize the databases running in AWS"""
    __do_init("init_one_aws_demo_db", namespace, None)
