#!/usr/bin/python3

import getopt
import os

from functions import *

#
# Load test data in a SQL file
#

assert sys.version_info >= (3, 7)

script_path = os.path.dirname(os.path.realpath(__file__))


def __usage():
    print("load.py [-l | -d] [-n namespace] [-a adaptor #] [-p] [-f filename.sql]\n")
    print("-d = AWS Docker/Kubernetes")
    print("-l = Local Docker")
    print("-n = Namespace in AWS Demo")
    print("-p = Run post-processing script (sets referential values)")
    print("-f = SQL file name")
    print("-a = Adaptor number:")
    print("     1 Asia ")
    print("     2 Europe West ")
    print("     3 North America ")
    print("     4 Europe East ")


def load_data(adaptor_number, file_name, local=True, post=False, namespace="default"):
    if adaptor_number == "1":
        app = "asia-postgres-db"
        cmd = postgresql_cmd(["-f", "/temp.sql"])
        header_file = script_path + "/../test-data/scripts/postgresql-header.sql"
        post_file = script_path + "/../test-data/scripts/postgresql-post.sql"

    elif adaptor_number == "2":
        app = "europe-west-sql-server-db"
        cmd = sqlserver_cmd(["-i", "temp.sql"])
        header_file = script_path + "/../test-data/scripts/sqlserver-header.sql"
        post_file = script_path + "/../test-data/scripts/sqlserver-post.sql"

    elif adaptor_number == "3":
        app = "north-america-postgres-db"
        cmd = postgresql_cmd(["-f", "/temp.sql"])
        header_file = script_path + "/../test-data/scripts/postgresql-header.sql"
        post_file = script_path + "/../test-data/scripts/postgresql-post.sql"

    elif adaptor_number == "4":
        app = "europe-east-mysql-db"
        cmd = ["sh", "-c", " ".join(mysql_cmd()) + " < /temp.sql"]
        header_file = script_path + "/../test-data/scripts/mysql-header.sql"
        post_file = script_path + "/../test-data/scripts/mysql-post.sql"

    else:
        print("ERROR: unknown adaptor number: " + str(adaptor_number))
        sys.exit(4)

    print("Loading " + file_name + " into adaptor #" + str(adaptor_number))

    # create temp.sql from the header, contents and body
    try:
        f = open("temp.sql", "w")
        try:
            f.write(read_file(header_file))
            f.write("\n")
            f.write(read_file(file_name))
            if post:
                f.write("\n")
                f.write(read_file(post_file))
        finally:
            f.close()

        if local:
            exec_command(["docker", "cp", "temp.sql", app + ":/temp.sql"])
            try:
                exec_command(["docker", "exec", app] + cmd)
            finally:
                exec_command(["docker", "exec", app, "rm", "temp.sql"])
        else:
            pod = k8s_get_pod_name(app, namespace)
            k8s_cp(pod, "temp.sql", "/temp.sql", namespace)
            try:
                k8s_exec(pod, cmd, namespace)
            finally:
                k8s_exec(pod, ["rm", "/temp.sql"], namespace)
    finally:
        # remove temp file
        for f in os.listdir("."):
            if f == "temp.sql":
                os.remove(f)


if __name__ == '__main__':
    try:
        opts, _ = getopt.getopt(sys.argv[1:], "dlpha:n:f:")
    except getopt.GetoptError as err:
        print("ERROR: " + err.msg + "\n")
        __usage()
        sys.exit(5)

    namespace = "default"
    demo = False
    local = False
    adaptor_number = None
    sql_file = None
    post = False

    for o, v in opts:
        if o == "-d":
            demo = True
            global_file = script_path + "/../aws_global.json"
        elif o == "-l":
            local = True
        elif o == "-a":
            adaptor_number = v
        elif o == "-n":
            namespace = v
        elif o == "-f":
            sql_file = v
        elif o == "-p":
            post = True
        elif o == "-h":
            __usage()
            sys.exit(0)

    if not (demo ^ local):
        print("ERROR: -l or -d must be specified (but not both)\n")
        __usage()
        sys.exit(1)

    if adaptor_number is None:
        print("ERROR: -a must be used to specify adaptor number\n")
        __usage()
        sys.exit(2)

    if sql_file is None:
        print("ERROR: -f must be used to specify the SQL file\n")
        __usage()
        sys.exit(3)

    load_data(adaptor_number, sql_file, local, post, namespace)
