#!/usr/bin/python3

import getopt
import json
import os
import sys

from functions import get_token, get_zone, get_domain, get_domain_version, post_outbound_acl

#
# Set the ACLs so 'Asia' zone is blocking employee data from going out to the other zones.
# - Then load employee_1a.sql and employee_1b.sql into the Asia  adaptor
# - Load employee_1a_ambiguous.sql and employee_1b_ambiguous.sql into any of the other adaptors
#   to create an ambiguous data exception
#
# Before running this make sure the YOUnite stack is running (locally or in AWS) and that the younite-api server is ready.
#
# To verify that the younite-api server is running, make a request to the /health endpoint from a browser.
#
# For example, if running locally:
#
# http://local.younite.org:8080/health
#


assert sys.version_info >= (3, 7)

def usage():
    print("Usage: setup-acls.py [-d | -l]")
    print("-d = AWS Docker/Kubernetes")
    print("-l = Local Docker")


try:
    opts, _ = getopt.getopt(sys.argv[1:], "dl")
except getopt.GetoptError as err:
    print("ERROR: " + err.msg + "\n")
    usage()
    sys.exit(1)

script_path = os.path.dirname(os.path.realpath(__file__))

global_file = script_path + "/../local_global.json"
namespace = "default"
demo = False
local = False

for o, _ in opts:
    if o == "-d":
        demo = True
        global_file = script_path + "/../aws_global.json"
    elif o == "-l":
        local = True
    elif o == "-h":
        usage()
        sys.exit(0)

if not (demo ^ local):
    usage()
    sys.exit(2)

try:
    gf = open(global_file, "r")
    json_global = json.load(gf)
    values = json_global.get("values")
    token_href = next((x.get("value") for x in values if x.get("key") == "token_href"), None)
    client_id = next((x.get("value") for x in values if x.get("key") == "client_id"), None)
    client_secret = next((x.get("value") for x in values if x.get("key") == "client_secret"), None)
    username = next((x.get("value") for x in values if x.get("key") == "dgs_username"), None)
    password = next((x.get("value") for x in values if x.get("key") == "dgs_password"), None)
    href = next((x.get("value") for x in values if x.get("key") == "href"), None)

    dgs_token = get_token(token_href, client_id, client_secret, username, password)
    if dgs_token is None:
        print("Either the YOUnite server is down or the credentials for the following user are invalid: dgs")
        sys.exit(3)

    asia_zone_uuid = get_zone(href, dgs_token, "Asia Headquarters")
    employee_domain_uuid = get_domain(href, dgs_token, "employee")
    employee_domain_version_uuid = get_domain_version (href, dgs_token, employee_domain_uuid, "1")

    response = post_outbound_acl(href, dgs_token,
                    asia_zone_uuid,
                    "00000000-0000-0000-0000-000000000000",
                    employee_domain_version_uuid,
                    "00000000-0000-0000-0000-000000000000",
                    "00000000-0000-0000-0000-000000000000",
                    False, True, True, True)

    print (response)

finally:
    # remove temp files
    for f in os.listdir("."):
        if f.startswith("credentials-") and f.endswith(".json"):
            os.remove(f)

print("Done")